#
# This file is public domain.
#

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'


# Deferred translation
def _(message):
    return message


# Languages we provide translations for, out of the box.
LANGUAGES = {
    'af': _('Afrikaans'),
    'ar': _('Arabic'),
    'ast': _('Asturian'),
    'az': _('Azerbaijani'),
    'bg': _('Bulgarian'),
    'be': _('Belarusian'),
    'bn': _('Bengali'),
    'br': _('Breton'),
    'bs': _('Bosnian'),
    'ca': _('Catalan'),
    'cs': _('Czech'),
    'cy': _('Welsh'),
    'da': _('Danish'),
    'de': _('German'),
    'el': _('Greek'),
    'en': _('English'),
    'en_AU': _('English (Australia)'),
    'en_CA': _('English (Canada)'),
    'en_GB': _('English (United Kingdom)'),
    'eo': _('Esperanto'),
    'es': _('Spanish'),
    'es-ar': _('Argentinian Spanish'),
    'es-mx': _('Mexican Spanish'),
    'es-ni': _('Nicaraguan Spanish'),
    'es-ve': _('Venezuelan Spanish'),
    'et': _('Estonian'),
    'eu': _('Basque'),
    'fa': _('Persian'),
    'fi': _('Finnish'),
    'fr': _('French'),
    'fy': _('Frisian'),
    'ga': _('Irish'),
    'gl': _('Galician'),
    'he': _('Hebrew'),
    'hi': _('Hindi'),
    'hr': _('Croatian'),
    'hu': _('Hungarian'),
    'ia': _('Interlingua'),
    'id': _('Indonesian'),
    'is': _('Icelandic'),
    'it': _('Italian'),
    'iu': _('Inuktitut'),
    'ja': _('Japanese'),
    'ka': _('Georgian'),
    'kk': _('Kazakh'),
    'km': _('Khmer'),
    'kn': _('Kannada'),
    'ko': _('Korean'),
    'ku': _('Kurdish'),
    'la': _('Latin'),
    'lb': _('Luxembourgish'),
    'lt': _('Lithuanian'),
    'lv': _('Latvian'),
    'mk': _('Macedonian'),
    'ml': _('Malayalam'),
    'mn': _('Mongolian'),
    'ms': _('Mayaly'),
    'my': _('Burmese'),
    'nb': _('Norwegian Bokmal'),
    'ne': _('Nepali'),
    'nds': _('Low German'),
    'nl': _('Dutch'),
    'nn': _('Norwegian Nynorsk'),
    'os': _('Ossetic'),
    'pa': _('Punjabi'),
    'pl': _('Polish'),
    'pms': _('Piedmontese'),
    'pt': _('Portuguese'),
    'pt_BR': _('Brazilian Portuguese'),
    'ro': _('Romanian'),
    'ru': _('Russian'),
    'sk': _('Slovak'),
    'sl': _('Slovenian'),
    'si': _('Sinhalese'),
    'sq': _('Albanian'),
    'sr': _('Serbian'),
    'sr-latn': _('Serbian Latin'),
    'sv': _('Swedish'),
    'sw': _('Swahili'),
    'ta': _('Tamil'),
    'te': _('Telugu'),
    'th': _('Thai'),
    'tl': _('Tagalog'),
    'tlh': _('Klingon'),
    'tr': _('Turkish'),
    'tt': _('Tatar'),
    'udm': _('Udmurt'),
    'uk': _('Ukrainian'),
    'ur': _('Urdu'),
    'vi': _('Vietnamese'),
    'zh_CN': _('Chinese (Simplified)'),
    'zh_HK': _('Chinese (Hong Kong)'),
    'zh-hans': _('Simplified Chinese'),
    'zh-hant': _('Traditional Chinese'),
    'zh_TW': _('Chinese (Taiwan)'),
}

del _
