/**
 * Ext.ux.form.SpinnerField.js
 *
 * Copyright (c) Damien Churchill 2010 <damoxc@gmail.com>
 *
 * This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
 * the additional special exception to link portions of this program with the OpenSSL library.
 * See LICENSE for more details.
 */

Ext.override(Ext.ux.form.SpinnerField, {
    onBlur: Ext.form.Field.prototype.onBlur,
});
