/*
 * ColumnTree styles
 */
.x-column-tree .x-tree-node {
    zoom:1;
}
.x-column-tree .x-tree-node-el {
    /*border-bottom:1px solid #eee; borders? */
    zoom:1;
}
.x-column-tree .x-tree-selected {
    background: #d9e8fb;
}
.x-column-tree  .x-tree-node a {
    line-height:18px;
    vertical-align:middle;
}
.x-column-tree  .x-tree-node a span{

}
.x-column-tree  .x-tree-node .x-tree-selected a span{
    background:transparent;
    color:#000;
}
.x-tree-col {
    float:left;
    overflow:hidden;
    padding:0 1px;
    zoom:1;
}

.x-tree-col-text, .x-tree-hd-text {
    overflow:hidden;
    -o-text-overflow: ellipsis;
    text-overflow: ellipsis;
    padding:3px 3px 3px 5px;
    white-space: nowrap;
    font:normal 11px arial, tahoma, helvetica, sans-serif;
}

.x-tree-headers {
    background: #f9f9f9 url(../../resources/images/default/grid/grid3-hrow.gif) repeat-x 0 bottom;
    cursor:default;
    zoom:1;
}

.x-tree-hd {
    float:left;
    overflow:hidden;
    border-left:1px solid #eee;
    border-right:1px solid #d0d0d0;
}

/*
 * FileUploadField component styles
 */
.x-form-file-wrap {
    position: relative;
    height: 22px;
}
.x-form-file-wrap .x-form-file {
    position: absolute;
    right: 0;
    -moz-opacity: 0;
    filter:alpha(opacity: 0);
    opacity: 0;
    z-index: 2;
    height: 22px;
    cursor: pointer;
}
.x-form-file-wrap .x-btn {
    position: absolute;
    right: 0;
    z-index: 1;
}
.x-form-file-wrap .x-form-file-text {
    position: absolute;
    left: 0;
    z-index: 3;
    color: #777;
}

/*
 * Spinner styles
 */
.x-form-spinner-proxy{
    /*background-color:#ff00cc;*/
}
.x-form-field-wrap .x-form-spinner-trigger {
    background: transparent url(../images/spinner.gif) no-repeat 0 0;
}

.x-form-field-wrap .x-form-spinner-overup{
    background-position:-17px 0;
}
.x-form-field-wrap .x-form-spinner-clickup{
    background-position:-34px 0;
}
.x-form-field-wrap .x-form-spinner-overdown{
    background-position:-51px 0;
}
.x-form-field-wrap .x-form-spinner-clickdown{
    background-position:-68px 0;
}


.x-trigger-wrap-focus .x-form-spinner-trigger{
    background-position:-85px 0;
}
.x-trigger-wrap-focus .x-form-spinner-overup{
    background-position:-102px 0;
}
.x-trigger-wrap-focus .x-form-spinner-clickup{
    background-position:-119px 0;
}
.x-trigger-wrap-focus .x-form-spinner-overdown{
    background-position:-136px 0;
}
.x-trigger-wrap-focus .x-form-spinner-clickdown{
    background-position:-153px 0;
}
.x-trigger-wrap-focus .x-form-trigger{
    border-bottom: 1px solid #7eadd9;
}

.x-form-field-wrap .x-form-spinner-splitter {
    line-height:1px;
    font-size:1px;
    background:transparent url(../images/spinner-split.gif) no-repeat 0 0;
    position:absolute;
    cursor: n-resize;
}
.x-trigger-wrap-focus .x-form-spinner-splitter{
    background-position:-14px 0;
}

.x-form-uxspinner .x-form-field-wrap {
    height: 24px;
}

/*
 * Ext JS Library 3.4.0
 * Copyright(c) 2006-2011 Sencha Inc.
 * licensing@sencha.com
 * http://www.sencha.com/license
 */
/* StatusBar - structure */
.x-statusbar .x-status-text {
    cursor: default;
/*
    height: 21px;
    line-height: 21px;
    padding: 0 4px;
*/
}
.x-statusbar .x-status-busy {
    padding-left: 25px !important;
    background: transparent no-repeat 3px 2px;
}

.x-toolbar div.xtb-text

.x-statusbar .x-status-text-panel {
    border-top: 1px solid;
    border-right: 1px solid;
    border-bottom: 1px solid;
    border-left: 1px solid;
    padding: 2px 8px 2px 5px;
}

/* StatusBar word processor example styles */

#word-status .x-status-text-panel .spacer {
    width: 60px;
    font-size:0;
    line-height:0;
}
#word-status .x-status-busy {
    padding-left: 25px !important;
    background: transparent no-repeat 3px 2px;
}
#word-status .x-status-saved {
    padding-left: 25px !important;
    background: transparent no-repeat 3px 2px;
}

/* StatusBar form validation example styles */

.x-statusbar .x-status-error {
    cursor: pointer;
    padding-left: 25px !important;
    background: transparent no-repeat 3px 2px;
}
.x-statusbar .x-status-valid {
    padding-left: 25px !important;
    background: transparent no-repeat 3px 2px;
}
.x-status-error-list {
    font: 11px tahoma,arial,verdana,sans-serif;
    position: absolute;
    z-index: 9999;
    border-top: 1px solid;
    border-right: 1px solid;
    border-bottom: 1px solid;
    border-left: 1px solid;
    padding: 5px 10px;
}
.x-status-error-list li {
    cursor: pointer;
    list-style: disc;
    margin-left: 10px;
}
.x-status-error-list li a {
    text-decoration: none;
}
.x-status-error-list li a:hover {
    text-decoration: underline;
}


/* *********************************************************** */
/* *********************************************************** */
/* *********************************************************** */


/* StatusBar - visual */

.x-statusbar .x-status-busy {
    background-image: url(../images/loading.gif);
}
.x-statusbar .x-status-text-panel {
    border-color: #99bbe8 #fff #fff #99bbe8;
}

/* StatusBar word processor example styles */

#word-status .x-status-text {
    color: #777;
}
#word-status .x-status-busy {
    background-image: url(../images/saving.gif);
}
#word-status .x-status-saved {
    background-image: url(../images/saved.png);
}

/* StatusBar form validation example styles */

.x-statusbar .x-status-error {
    color: #C33;
    background-image: url(../images/exclamation.gif);
}
.x-statusbar .x-status-valid {
    background-image: url(../images/accept.png);
}
.x-status-error-list {
    border-color: #C33;
}
.x-status-error-list li a {
    color: #15428B;
}
